use glow::HasContext;
use std::path::Path;


/// SpriteSheet Struct for Representing a Spritesheet
/// feel free to clone
#[derive(Clone)]
pub struct SpriteSheet
{
    texture_id: glow::NativeTexture,
    // NOTE where the spritesheet is saved
    path:       String,
    cuts_x:     u32,
    // NOTE possibly making it check vertically
    //cuts_y:   u32,
    // NOTE maybe add an offset for each sprite
    // offsets: Vec<cgmath::Vec2<f32>>
    // NOTE Hold frames, if 0 it means it will be displayed for only 1 frame, 1 means it will be held for 2 etc
    // hold:    Vec<u32>
}

/// load texture from a filepath and return the ID
/// NOTE this function does not check if the texture has already loaded
fn texture_from_file(path: &str, gl: &glow::Context) -> glow::NativeTexture
    {

        let texture_id;
        unsafe {
            texture_id = gl.create_texture().unwrap();

            let img = image::open(&Path::new(&path)).expect("texture failed to load");
            let img = img.flipv();

            let format = match img {
                image::DynamicImage::ImageLuma8(_)  => glow::RED,
                image::DynamicImage::ImageLumaA8(_) => glow::RG,
                image::DynamicImage::ImageRgb8(_)   => glow::RGB,
                image::DynamicImage::ImageRgba8(_)  => glow::RGBA,
                _ => panic!("unknown image format at {path}")
            };

            let data = img.as_bytes();

            gl.bind_texture(glow::TEXTURE_2D, Some(texture_id));
            gl.tex_image_2d(glow::TEXTURE_2D, 0, format as i32, img.width() as i32, img.height() as i32,
                            0, format, glow::UNSIGNED_BYTE, Some(&data));
            gl.generate_mipmap(glow::TEXTURE_2D);


            gl.tex_parameter_i32(glow::TEXTURE_2D, glow::TEXTURE_WRAP_S, glow::REPEAT as i32);
            gl.tex_parameter_i32(glow::TEXTURE_2D, glow::TEXTURE_MAG_FILTER, glow::NEAREST as i32);
            gl.tex_parameter_i32(glow::TEXTURE_2D, glow::TEXTURE_MAG_FILTER, glow::NEAREST_MIPMAP_NEAREST as i32);

        }

        texture_id
}
